#define G_LOG_DOMAIN "gpod-manager"

#include "config.h"

#include "gpod-manager.h"

#include <libgrss.h>
#include <tracker-sparql.h>

#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <glib/gi18n.h>

/**
 * SECTION:gpod-manager
 * @title: GpodManager
 * @short_description: manages feeds and saving of information to and from the
 *                     tracker database.
 * @include: gpod-manager.h
 *
 * The #GpodManager is object to manage the fetching, saving, and retrevial of
 * feeds and their items.
 **/

typedef struct _GpodManagerPrivate
{
    GList *feeds;
} GpodManagerPrivate;

struct _GpodManager
{
    GObject parent;
};

G_DEFINE_TYPE_WITH_PRIVATE (GpodManager, gpod_manager, G_TYPE_OBJECT)

/*
 *******************************************************************************
 * GList compare functions
 *******************************************************************************
 */
gint
feeds_search_title (gconstpointer a,
                    gconstpointer b)
{
    GrssFeedChannel *feed = GRSS_FEED_CHANNEL (a);
    gchar *search = (gchar *) b;

    g_return_if_fail (GRSS_IS_FEED_CHANNEL (feed));

    return strcmp (grss_feed_channel_get_title (feed), search);
}

gint
feeds_search_source (gconstpointer a,
                     gconstpointer b)
{
    GrssFeedChannel *feed = GRSS_FEED_CHANNEL (a);
    gchar *source = (gchar *) b;

    g_return_if_fail (GRSS_IS_FEED_CHANNEL (feed));

    return strcmp (grss_feed_channel_get_source (feed), source);
}

/*
 *******************************************************************************
 * Internal feed actions
 *******************************************************************************
 */
static void
add_feed (GpodManager     *self,
          GrssFeedChannel *feed)
{
    g_return_if_fail (GPOD_IS_MANAGER (self));
    g_return_if_fail (GRSS_IS_FEED_CHANNEL (feed));

    GpodManagerPrivate *priv = gpod_manager_get_instance_private (self);

    g_debug ("Feed added: \"%s\"\n", grss_feed_channel_get_title (feed));

    priv->feeds = g_list_append (priv->feeds, feed);
}

static void
add_feed_with_source (GpodManager     *self,
                      GrssFeedChannel *feed)
{
    g_return_if_fail (GPOD_IS_MANAGER (self));
    g_return_if_fail (GRSS_IS_FEED_CHANNEL (feed));

    GpodManagerPrivate *priv = gpod_manager_get_instance_private (self);
    gboolean fetched;
    GError *error;

    fetched = grss_feed_channel_fetch (feed, &error);

    if (fetched == FALSE)
    {
        g_debug ("Failed to populate feed: \"%s\"", grss_feed_channel_get_source (feed));
        return;
    }

    add_feed (self, feed);
}

static GList *
get_feeds (GpodManager *self)
{
    g_return_if_fail (GPOD_IS_MANAGER (self));

    GpodManagerPrivate *priv = gpod_manager_get_instance_private (self);

    return priv->feeds;
}

static GrssFeedChannel *
find_feed_by_title (GpodManager *self,
                    gchar       *title)
{
    g_return_if_fail (GPOD_IS_MANAGER (self));

    GpodManagerPrivate *priv = gpod_manager_get_instance_private (self);

    g_list_find_custom (priv->feeds, title, feeds_search_title);
}

static GrssFeedChannel *
find_feed_by_source (GpodManager *self,
                     gchar       *source)
{
    g_return_if_fail (GPOD_IS_MANAGER (self));

    GpodManagerPrivate *priv = gpod_manager_get_instance_private (self);

    g_list_find_custom (priv->feeds, source, feeds_search_source);
}

static void
fetch_feed_items (GrssFeedChannel *feed,
                  GAsyncResult    *result,
                  gpointer         user_data)
{
    g_return_if_fail (GRSS_IS_FEED_CHANNEL (feed));
    g_return_if_fail (GPOD_IS_MANAGER (user_data));

    GError *error = NULL;
    GList *items;
    GList *iter;

    items = grss_feed_channel_fetch_all_finish (GRSS_FEED_CHANNEL (feed),
                                                result,
                                                &error);

    if (items == NULL)
    {
        g_print ("%s\n", error->message);
    }
    else
    {
        for (iter = items; iter; iter = g_list_next (iter))
        {
            GrssFeedItem *item = (GrssFeedItem*) iter->data;
            g_print ("%s\n", grss_feed_item_get_title (item));
        }
    }
}

static void
get_feed_items (GpodManager     *self,
                GrssFeedChannel *feed)
{
    grss_feed_channel_fetch_all_async (feed, fetch_feed_items, self);
}

/*
 * gpod_manager_finalize:
 * @object: An instance of #GpodManager
 *
 * Does any last minute clean-ups when the class ends.
 */
static void
gpod_manager_finalize (GObject *object)
{
    GpodManager *self = GPOD_MANAGER (object);
    GpodManagerPrivate *priv = gpod_manager_get_instance_private (self);

    g_list_free_full (priv->feeds, g_object_unref);
    priv->feeds = NULL;

    G_OBJECT_CLASS (gpod_manager_parent_class)->finalize(object);
}

/*
 * gpod_manager_init:
 * @self: An instance of #GpodManager
 *
 * Initializes various aspects of the #GpodManager. Anything the #GpodManager
 * would need before becoming presented should be done here.
 */
static void
gpod_manager_init (GpodManager *self)
{
    GpodManagerPrivate *priv = gpod_manager_get_instance_private (self);

    priv->feeds = NULL;
}

/*
 * gpod_manager_class_init:
 * @klass: An instance of #GpodManagerClass
 *
 * Initializes the various virtural functions in #GpodManagerClass. In
 * addtition it connects the various #GtkWidgets from the gresource file using
 * gtk_widget_class_set_template_from_resource().
 */
static void
gpod_manager_class_init (GpodManagerClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);

    object_class->finalize = gpod_manager_finalize;
}

/**
 * gpod_manager_new:
 *
 * Creates a new #GpodManager, a custom widget to show all feeds user is
 * subscribed to.
 *
 * ## Usage
 * |[
 * GpodManager *manager;
 *
 * manager = gpod_manager_new ();
 * ]|
 *
 * Returns: #GpodManager
 */
GpodManager *
gpod_manager_new ()
{
    return g_object_new (GPOD_TYPE_MANAGER,
                         NULL);
}

void
gpod_manager_add_feed_by_url (GpodManager *self,
                              gchar       *feed_url)
{
    GrssFeedChannel *feed;

    g_return_if_fail (GPOD_IS_MANAGER (self));

    if (feed_url == NULL || feed_url == "")
        return;

    feed = grss_feed_channel_new_with_source (feed_url);

    add_feed_with_source (self, feed);
}

GList *
gpod_manager_get_feeds (GpodManager *self)
{
    return get_feeds (self);
}
